/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-COM-2                 **/
/**                                                                       **/
/**                             Version 2.A.                              **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C"                           **/
/**                                                                       **/
/**             Project start    : 02.04.1997                             **/
/**             Last change      : 02.04.1997                             **/
/**                                                                       **/
/**                         Author : H.K.                                 **/
/**                         Status : O.K.                                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.                 **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1997                           **/
/**                                                                       **/
/***************************************************************************/

/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) und C (project-file). They can also be set in this source file:

   Then the following #defines have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included (default is PC library).

   If you work with Windows, this is detected by the compiler. No #defines have
   to be included.
*/

#include "m032_lib.h"

 const STATIC ushort channels[2] = {0x02, 0x00};

 const STATIC byte M032_INTERRUPT[7][3] = {{M032_C1_FALSE,M032_C2_FALSE,M032_C3_FALSE},/* Kein Interrupt */
                                           {M032_C1_FALSE,M032_C2_TRUE, M032_C3_FALSE}, /* IRQ-A          */
                                           {M032_C1_FALSE,M032_C2_TRUE, M032_C3_TRUE }, /* IRQ-F          */
                                           {M032_C1_FALSE,M032_C2_FALSE,M032_C3_TRUE }, /* IRQ-B          */
                                           {M032_C1_TRUE, M032_C2_TRUE, M032_C3_TRUE }, /* IRQ-E          */
                                           {M032_C1_TRUE, M032_C2_TRUE, M032_C3_FALSE}, /* IRQ-C          */
                                           {M032_C1_TRUE, M032_C2_FALSE,M032_C3_TRUE }};/* IRQ-D          */

#define CHANNEL channels[channel]      /* Channels A..B                    */


						 /* Static Arrays                    */

						 /* SCC-Type :                       */
						 /*           0 = 8530               */
						 /*           1 = 85130              */
						 /*           2 = 85230              */
 STATIC ushort m032scc_type[ANZ_MOD];

						 /* Interrupt-Select-Register :      */
						 /*           0 = -                  */
						 /*           1 = IRQ-A / INT-1      */
						 /*           2 = IRQ-B / INT-3      */
						 /*           3 = IRQ-E / INT-4      */
						 /*           4 = IRQ-C / INT-5      */
						 /*           5 = IRQ-D / INT-7      */

// STATIC short  m032intsel[ANZ_MOD];

 STATIC ushort m032clink[ANZ_MOD] [2]; /* C-Link type of each channel */

 STATIC ushort m032quartz[ANZ_MOD];    /* QUARTZ Frequency of module       */
						                     /* e.g. 25000 = 25000 kHz = 25 MHz       */

 STATIC byte   tmtctr[ANZ_MOD][8];


#if defined (__DLL__) && !defined (__OS2__) /* if a module DLL shall be generated */


#include <windows.h>

HANDLE   hInst;

#pragma argsused

int far pascal LibMain(hInstance, wDataSeg, wHeapSize, lpszCmdLine)
HANDLE  hInstance;                     /* Instance of DLL                  */
WORD    wDataSeg;                      /* Data segment                     */
WORD    wHeapSize;                     /* Size of Heap                     */
LPSTR   lpszCmdLine;                   /* Pointer to command line          */

{
 hInst = hInstance;

 if(wHeapSize > 0) UnlockData(0);

 return(1);
}

#endif  /* __DLL__ */


/* Delay between accesses to SCC */
void m032_delay(void)
{
 mlx_out_port(0x3F,0);        /* 6 accesses to address 0x3F (unused) */
 mlx_out_port(0x3F,0);
 mlx_out_port(0x3F,0);
 mlx_out_port(0x3F,0);
 mlx_out_port(0x3F,0);
 mlx_out_port(0x3F,0);
}


/***************************************************************************/
/* Init library with EEPROM-values                                         */
/*                                                                         */
/* This procedure checks the module type in slot "micro_slot", reads the   */
/* EEPROM values and stores them in static library-variables.              */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/***************************************************************************/
MLXAPI void EXPORT m032_bib_startup(byte micro_slot)
{
 ushort w;
 short  endm, m;

				       /* Check, whether module extender   */
				       /* is placed at slot 1              */
 if(lobyte(mlx_read_eeprom(1, 0)) == 0x29) endm = ANZ_MOD;
 else endm = 5;                        /* 4 slots                          */

 for(m = 1; m < endm; m++)             /* Check all slots                  */
	if(lobyte(mlx_read_eeprom(m, 0)) == 0x20)
	  {                                 /* Correct module = 32 = 0x20         */

		w = mlx_read_eeprom(m, 2);
		m032scc_type[m] = lobyte(w) & 0x0F;

		w = mlx_read_eeprom(m, 3);       /* get quartz frequency of module   */
		m032quartz[m]  = ((w & 0x000F))       *    1;
		m032quartz[m] += ((w & 0x00F0) >>  4) *   10;
		m032quartz[m] += ((w & 0x0F00) >>  8) *  100;
		m032quartz[m] += ((w & 0xF000) >> 12) * 1000;
		m032quartz[m] *= 10;
      w = mlx_read_eeprom(m, 6);       /* get C-Link-Type of channel A */
      m032clink[m][0] = lobyte(w);

      w = mlx_read_eeprom(m, 7);       /* get C-Link-Type of channel B */
      m032clink[m][1] = lobyte(w);

	  }
 micro_slot = micro_slot;
}


/***************************************************************************/
/* Hardware-Reset SCC                                                      */
/*                                                                         */
/* This procedure forces a hardware interrupt on SCC                       */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/***************************************************************************/

MLXAPI void EXPORT m032_scc_reset(byte micro_slot)
{                                      /* SCC-Force Hardware-Reset         */
 m032_delay();
 mlx_out_port(MBASIS, 0x09);
 m032_delay();
 mlx_out_port(MBASIS, 0xC0);
 m032_delay();
}


/***************************************************************************/
/* Initialize one Channel of the SCC                                       */
/*                                                                         */
/* This function initializes one channel of the SCC.                       */
/* The interrupt vector is set to the SCC in WR 2, Bit 6..7.               */
/* The parameter "int_mode" has the same function as WR 1 of a SCC,        */
/* only the lower 5 bits are used.                                         */
/* The parameter "mode" selects the clock-mode for one channel.            */
/* It is possible to connect a module-clock (PCLK, baseclk 1..4, etc.) to  */
/* generate the transmit and receive clocks, the clock can also be delive- */
/* red by another channel using the RTS-line of the interface or a clock   */
/* can be delivered to another interface via the RTS-line.                 */
/* It is important not to change the order of the SCC-register program. !  */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B                                   */
/*                                                                         */
/* mode       : 0 bis 7                                                    */
/*                                                                         */
/* int_mode   : =Write Register 1 of a SCCs channel (WR 1, lower 5 bits)   */
/*                 7 6 5 4 3 2 1 0                                         */
/*                 | | | | | | | 0--- Ext./Status Master Interrupt Disable */
/*                 | | | | | | | 1--- Ext./Status Master Interrupt Enable  */
/*                 | | | | | | 0----- Transmit Buffer Empty Inter. Disable */
/*                 | | | | | | 1----- Transmit Buffer Empty Inter. Enable  */
/*                 | | | | | 0------- no Interrupt on Parity not Matching  */
/*                 | | | | | 1------- Parity not Matching Is Special Cond. */
/*                 | | | 0-0--------- Receive Interupts Disabled           */
/*                 | | | 0-1--------- Receive Interupt on First Charac. or */
/*                 | | | | |          Special Condition                    */
/*                 | | | 1-0--------- Interrupt on All Receive Charac. or  */
/*                 | | |              Special Condition                    */
/*                 | | | 1-1--------- Receive Interupt on Special Condit.  */
/*                 0 0 0 ------------ reserved                             */
/*                                                                         */
/* int_x_mode : =Write Register 15 of a SCCs channel (WR 15)               */
/*               External/Status Interrupt Control                         */
/*                 7 6 5 4 3 2 1 0                                         */
/*                 | | | | | | | 0--- reserved for ESCC                    */
/*                 | | | | | | 0----- Zero Count Interrupt Disable         */
/*                 | | | | | | 1----- Zero Count Interrupt Enable          */
/*                 | | | | | 0------- SDLC Fifo Disable (only ESCC & CMOS) */
/*                 | | | | | 1------- SDLC Fifo Enable (only ESCC & CMOS)  */
/*                 | | | | 0--------- DCD (=CTS) Interrupt Disable         */
/*                 | | | | 1--------- DCD (=CTS) Interrupt Enable          */
/*                 | | | 0----------- reserved                             */
/*                 | | 0------------- CTS (=DCD) Interrupt Disable         */
/*                 | | 1------------- CTS (=DCD) Interrupt Enable          */
/*                 | 0--------------- Transmit Underrun Interrupt Disable  */
/*                 | 1--------------- Transmit Underrun Interrupt Enable   */
/*                 0----------------- Break/Abort Interrupt Disable        */
/*                 1----------------- Break/Abort Interrupt Enable         */
/*                                                                         */
/* timeconst  : SCC-Timeconst. of BRG (baudrate generator)                 */
/*                                                                         */
/* use_brg    : if use_brg == 0, the baudrate-generator is not used        */
/*                                                                         */
/* databits   : 5, 6, 7 or 8 Databits (transmit and receive)               */
/*                                                                         */
/* stopbits   : 0=no stopbits, 1=1 stopbit, 2=1,5 stopbits, 3=2 stopbits   */
/*                                                                         */
/* parity_type: 0=no parity , 1=odd parity, 2=even parity type             */
/*                                                                         */
/* Result     : if(> 0) Error                                              */
/***************************************************************************/
MLXAPI byte EXPORT m032_init_scc(byte micro_slot, byte channel,
			  byte mode, byte int_mode, byte int_x_mode,
			  ushort timeconst, byte use_brg, byte clkmode,
			  byte databits, byte stopbits, byte parity_type)
{
 byte usart;
 byte rcvcnt;
 ushort sccadr;
 union
	{
	 byte lo_hi[2];
	 ushort i;
	}timec;

 sccadr = MBASIS + CHANNEL;

 timec.i= timeconst;


 /**************************************************************************/
 /*      STAGE 1. Modes and Constants                                      */
 /**************************************************************************/

						 /************************************/
						 /* WR 9 Master Interrupt Control    */
						 /************************************/
 m032_delay();
 mlx_out_port(sccadr,  9);
 m032_delay();
 switch((int)(channel & 0x01))
   {                                   /* Reset Channel A of SCC           */
    case 0: mlx_out_port(sccadr, 0x80);
            m032_delay();
	    break;
						                     /* Reset Channel B of SCC           */
    case 1: mlx_out_port(sccadr, 0x40);
            m032_delay();
	    break;
    default: return(1);
   }


				       /************************************/
				       /* WR 4 Transm./Rec. Misc. & Modes  */
 usart = 0;                            /************************************/

 if(parity_type > 0)
   {
	 usart = usart | 1;                 /* Parity enabled = Bit 0 = 1       */
				                           /* Even Parity    = Bit 1 = 1       */
    if(parity_type == 2) usart = usart | 2;
   }

 switch(stopbits)
   {                                   /* Set Stop-Bits   = Bit 3 & 2      */
    case 0: break;
				                           /* 1 Stop-Bit      =     0   1      */
    case 1: usart = usart | 0x04; break;
				                           /* 1,5 Stop-Bit    =     1   0      */
    case 2: usart = usart | 0x08; break;
				                           /* 2 Stop-Bits     =     1   1      */
	 case 3: usart = usart | 0x0C; break;

    default: return(1);
   }
				                           /* Bit 5 & 4 ignored in async. mode */

 switch((int)clkmode)
   {
				                           /* Clock Mode      = Bit 7 & 6      */
    case  1: usart = usart | 0x00;     /* x1   (synchron.)=     0   0      */
	     break;
    case 16: usart = usart | 0x40;     /* x16 (asynchron.)=     0   1      */
	     break;
	 case 32: usart = usart | 0x80;     /* x32 (asynchron.)=     1   0      */
	     break;
    case 64: usart = usart | 0xC0;     /* x64 (asynchron.)=     1   1      */
	     break;
    default: return(1);
   }
 mlx_out_port(sccadr,  4);             /* WR 4                             */
 m032_delay();

 if ((mode == 3) || (mode == 5) || (mode == 0)) usart = usart & 0x3F;
 else
   if (mode == 1) usart = usart & 0x3F + 0x40;

 mlx_out_port(sccadr, usart);
 m032_delay();


				       /************************************/
				       /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);             /* Select IW/REQ (optional)         */
 m032_delay();
 mlx_out_port(sccadr, 0x00);
 m032_delay();


				       /************************************/
				       /* WR 2 Interrupt Vector            */
				       /************************************/

 mlx_out_port(sccadr,  2);             /* Interrupt Vector = SCC_NUM       */
 m032_delay();
 mlx_out_port(sccadr, (channel / 2) << 6);
 m032_delay();


						 /************************************/
				       /* WR 3 Receive Parameters & Contr. */
				       /************************************/
 usart = 0;
 switch (databits)
   {                                   /* No. of databits = Bit 7 & 6      */
    case 5: break;                     /*               5 =     0   0      */
				                           /*               6 =     1   0      */
    case 6: usart = usart | 0x80; break;
				                           /*               7 =     0   1      */
    case 7: usart = usart | 0x40; break;
				                           /*               8 =     1   1      */
    case 8: usart = usart | 0xC0; break;

    default: return(1);
   }
				                           /* Bit 5=0                          */
				                           /* Bit 4-2 ignored in async. mode   */
				                           /* Bit 1=0 SYNC Character disabl.   */
				                           /* Bit 0=0 Receiver disable         */

 mlx_out_port(sccadr,  3);             /* WR 3  Receive Parameters & Contr.*/
 m032_delay();
 mlx_out_port(sccadr, usart);
 m032_delay();

 rcvcnt = usart;                       /* store WR 3 settings in rcvcnt    */

						 /************************************/
				       /* WR 5 Transmit Parameters & Contr.*/
				       /************************************/


 usart = usart >> 1;                   /* Shift WR3 data one bit right     */

 mlx_out_port(sccadr,  5);             /* WR 5  Transm. Parameters & Contr.*/
 m032_delay();
 mlx_out_port(sccadr, usart);
 m032_delay();
				                           /* store WR 5 settings for this ch. */
 tmtctr[micro_slot][channel] = usart;


						 /************************************/
				       /* WR 6 not used in asynchr. mode   */
				       /************************************/

				       /************************************/
				       /* WR 7 is only used by the ESCC    */
				       /************************************/

				       /************************************/
				       /* WR 8 is the TRANSMIT BUFFER      */
				       /************************************/

				       /************************************/
						 /* WR 9 Master Interrupt Control    */
				       /************************************/

 mlx_out_port(sccadr,  9);             /* WR 9 Master Interrupt Control    */
 m032_delay();
 mlx_out_port(sccadr, 0x00);
 m032_delay();




				       /************************************/
				       /* WR 10 Misc. Transmitter/Receiver */
				       /************************************/
 mlx_out_port(sccadr, 10);             /* WR 10 Misc. Transmitter/Receiver */
 m032_delay();
 mlx_out_port(sccadr, 0x00);
 m032_delay();




 if(use_brg == 0x00) {
   mlx_out_port(sccadr, 0x05);         /* baudrate-generator not used      */
				                           /* transmit and receive             */
				                           /* must be clocked from RTxC-pin    */
   m032_delay();
   }
 else {
   switch (mode)
     {
      case 0: {
               mlx_out_port(sccadr+8+(channel*5),0);/* Port A/B1 = 0, A/B2 = 0, A/B3 = 0 */
               mlx_out_port(sccadr+9+(channel*5),0);
               mlx_out_port(sccadr+10+(channel*5),0);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x28);           /* TRxC is input, Transmit Clock =
                                                       Receive Clock = TRxC */

               m032_delay();
               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x60);
               m032_delay();
               break;
              }
      case 1: {
               mlx_out_port(sccadr+8+(channel*5),1);/* Port A/B1 = 1, A/B2 = 0, A/B3 = 0 */
               mlx_out_port(sccadr+9+(channel*5),0);
               mlx_out_port(sccadr+10+(channel*5),0);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x50);           /* TRxC is input, Transmit Clock =
                                                       Receive Clock = BRG Output */
               m032_delay();
               mlx_out_port(sccadr, 12);            /* Time-Constant (Low-Byte)         */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[0]);
               m032_delay();

               mlx_out_port(sccadr, 13);            /* Time-Constant (High-Byte)        */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[1]);
               m032_delay();

               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x62);
               m032_delay();
               break;
              }
      case 2: {
               mlx_out_port(sccadr+8+(channel*5),0);/* Port A/B1 = 0, A/B2 = 1, A/B3 = 0 */
               mlx_out_port(sccadr+9+(channel*5),1);
               mlx_out_port(sccadr+10+(channel*5),0);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x00);           /* TRxC is input, Transmit Clock =
                                                       Receive Clock = RTxC */
               m032_delay();
               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x60);
               m032_delay();
               break;
              }
      case 3: {
               mlx_out_port(sccadr+8+(channel*5),1);/* Port A/B1 = 1, A/B2 = 1, A/B3 = 0 */
               mlx_out_port(sccadr+9+(channel*5),1);
               mlx_out_port(sccadr+10+(channel*5),0);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x00);           /* TRxC is input, Transmit Clock =
                                                       Receive Clock = RTxC */
               m032_delay();
               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x60);
               m032_delay();
               break;
              }
      case 5: {
               mlx_out_port(sccadr+8+(channel*5),1);/* Port A/B1 = 1, A/B2 = 0, A/B3 = 1 */
               mlx_out_port(sccadr+9+(channel*5),0);
               mlx_out_port(sccadr+10+(channel*5),1);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x55);           /* TRxC is output = Transmit Clock,
                                                       Transmit Clock = Receive Clock =
                                                       BRG Out */
               m032_delay();
               mlx_out_port(sccadr, 12);            /* Time-Constant (Low-Byte)         */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[0]*16);
               m032_delay();
               mlx_out_port(sccadr, 13);            /* Time-Constant (High-Byte)        */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[1]*16);
               m032_delay();
               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x62);
               m032_delay();
               break;
              }
      case 7: {
               mlx_out_port(sccadr+8+(channel*5),1);/* Port A/B1 = 1, A/B2 = 1, A/B3 = 1 */
               mlx_out_port(sccadr+9+(channel*5),1);
               mlx_out_port(sccadr+10+(channel*5),1);

               mlx_out_port(sccadr, 11);            /* WR 11 Clock-Control              */
               m032_delay();
               mlx_out_port(sccadr,0x54);           /* TRxC is output = XTal Out,
                                                       Transmit Clock = Receive Clock =
                                                       BRG Out */
               m032_delay();
               mlx_out_port(sccadr, 12);            /* Time-Constant (Low-Byte)         */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[0]);
               m032_delay();
               mlx_out_port(sccadr, 13);            /* Time-Constant (High-Byte)        */
               m032_delay();
               mlx_out_port(sccadr, timec.lo_hi[1]);
               m032_delay();
               mlx_out_port(sccadr, 14);            /* WR 14 Miscellaneous Control      */
               m032_delay();
               mlx_out_port(sccadr, 0x62);
               m032_delay();
               break;
              }

      default: return(1);
     }
   }


 /**************************************************************************/
 /*      STAGE 2. Enables                                                  */
 /**************************************************************************/

				       /************************************/
				       /* WR 3 Receive Parameters & Contr. */
				       /************************************/

 mlx_out_port(sccadr,  3);             /* now Enable Receiver (Bit 0=1)    */
 m032_delay();
 mlx_out_port(sccadr, rcvcnt | 1);
 m032_delay();


						 /************************************/
				       /* WR 5 Transmit Parameters & Contr.*/
				       /************************************/

 mlx_out_port(sccadr,  5);             /* now Enable Transmitter (Bit 3=1) */
 m032_delay();
 tmtctr[micro_slot][channel] = tmtctr[micro_slot][channel] | 0x08;
 mlx_out_port(sccadr, tmtctr[micro_slot][channel] | 0x82);
 m032_delay();


				       /************************************/
				       /* WR 0 Command Register            */
				       /************************************/

						 /* WR 0 Init., Reset TxCRC          */
 mlx_out_port(sccadr, 0x80);
 m032_delay();


				       /************************************/
				       /* WR 14 Misc. Control              */
				       /************************************/

 if ((mode == 0) || (mode == 2) || (mode == 3)) {
   mlx_out_port(sccadr, 14);             /* WR 14, Enable Baudrate-Generator */
   m032_delay();
   mlx_out_port(sccadr, 0x00);
   m032_delay();
   }
 else {
   if ((mode == 1) || (mode == 5) || (mode == 7)) {
     mlx_out_port(sccadr, 14);           /* WR 14, Enable Baudrate-Generator */
     m032_delay();
     mlx_out_port(sccadr, 0x03);
     m032_delay();
     }
 }



				       /************************************/
						 /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);               /* Select if DMA (optional)         */
 m032_delay();
 mlx_out_port(sccadr, 0x00);
 m032_delay();


 /**************************************************************************/
 /*      STAGE 3. Interrupt Enables                                        */
 /**************************************************************************/

				       /************************************/
				       /* WR 15 Ext./Status Interrupt Con. */
						 /************************************/

 mlx_out_port(sccadr,  15);              /* Ext./Status Interrupt Enable     */
 m032_delay();
 mlx_out_port(sccadr, int_x_mode);
 m032_delay();


				       /************************************/
				       /* WR 0 Command Register            */
				       /************************************/

				       /* Reset Ext./Status (2x)           */
 mlx_out_port(sccadr, 0x10);
 m032_delay();

 mlx_out_port(sccadr, 0x10);
 m032_delay();


				       /************************************/
				       /* WR 1 Interrupt Enable            */
				       /************************************/

 mlx_out_port(sccadr,  1);               /* Enable receive, transmit and     */
				                             /* external interrupt master        */
 m032_delay();
 mlx_out_port(sccadr, int_mode & 0x1F);
 m032_delay();


				       /************************************/
						 /* WR 9 Master Interrupt Control    */
				       /************************************/

 mlx_out_port(sccadr,  9);               /* WR 9,                            */
				                             /* MIE = 1, Modify Vector V1-V3     */
 m032_delay();
 if(int_mode > 0) {
   mlx_out_port(sccadr, 0x09);
   m032_delay();
   }
				                             /* MIE = 0 ..., no interrupt        */
 else {
   mlx_out_port(sccadr, 0x01);
   m032_delay();
   }

 return(0);
}



/***************************************************************************/
/* Read Status of Transmit-Buffer                                          */
/*                                                                         */
/* This function reads the status of one channels transmit-buffer-empty    */
/* bit.                                                                    */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result = 1 : Transmit Buffer empty (TBE)                                */
/*                                                                         */
/* Result = 0 : last Character is still in Buffer                          */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_tbe(byte micro_slot, byte channel)
{
 byte tbe;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

						 /* Reset External Status Interrupt  */
 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 tbe = mlx_in_port(sccadr);
 m032_delay();

 return((tbe & 0x04) >> 2);
}



/***************************************************************************/
/* Read Status of Receive-Buffer                                           */
/*                                                                         */
/* This function reads the status of one channels receive-buffer-full bit. */
/* bit.                                                                    */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B												*/
/*                                                                         */
/* Result = 1 : Receive Buffer full (RBF)                                  */
/*                                                                         */
/* Result = 0 : no Character received                                      */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_rbf(byte micro_slot, byte channel)
{
 byte rbf;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;
						 /* Reset External Status Interrupt  */
 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 rbf = mlx_in_port(sccadr);
 m032_delay();

 return(rbf & 0x01);
}



/***************************************************************************/
/* Read Status of CTS                                                      */
/*                                                                         */
/* This function reads the status of one channels cts-line.                */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result = 1 : CTS = 1                                                    */
/*                                                                         */
/* Result = 0 : CTS = 0                                                    */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_cts(byte micro_slot, byte channel)
{
 byte cts;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 cts = mlx_in_port(sccadr);
 m032_delay();

 return((cts & 0x20) >> 5);
}



/***************************************************************************/
/* Read Status of DCD                                                      */
/*                                                                         */
/* This function reads the status of one channels dcd-line.                */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result = 1 : DCD = 1                                                    */
/*                                                                         */
/* Result = 0 : DCD = 0                                                    */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_dcd(byte micro_slot, byte channel)
{
 byte dcd;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 dcd = mlx_in_port(sccadr);
 m032_delay();

 return((dcd & 0x08) >> 3);
}

/***************************************************************************/
/* Read Status of Ri                                                      */
/*                                                                         */
/* This function reads the status of one channels Ri-line.                */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result = 1 : Ri = 1                                                    */
/*                                                                         */
/* Result = 0 : Ri = 0                                                    */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_ri(byte micro_slot, byte channel)
{
 byte ri;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 ri = mlx_in_port(sccadr);
 m032_delay();

 return((ri & 0x10) >> 4);
}


/***************************************************************************/
/* Read Status of DSR                                                      */
/*                                                                         */
/* This function reads the status of one channels dsr-line.                */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result = 1 : DSR = 1                                                    */
/*                                                                         */
/* Result = 0 : DSR = 0                                                    */
/***************************************************************************/
MLXAPI byte EXPORT m032_scc_DSR(byte micro_slot,byte channel)
{
 byte dsr;
 ushort sccadr;

 sccadr = MBASIS - CHANNEL + 10;

 m032_delay();
 dsr = mlx_in_port(sccadr);
 m032_delay();

 return(dsr & 1);
}

/***************************************************************************/
/* Transmit Character                                                      */
/*                                                                         */
/* The SCC must be initialized before !                                    */
/* Before transmitting the status of the trasnmit-buffer (TBE) is checked. */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* character  : Transmit Character                                         */
/*                                                                         */
/* Result     : 0=character sent, 1=transmit buffer not empty              */
/***************************************************************************/
MLXAPI byte EXPORT m032_transmit_scc_character(byte micro_slot, byte channel,
					byte character)
{
 byte tbe;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

 m032_delay();
 mlx_out_port(sccadr, 0x10);           /* Reset External Status Interrupt  */
 m032_delay();
 tbe = mlx_in_port(sccadr);            /* Transmit Buffer Empty ?          */
 m032_delay();

 if((tbe & 0x04) >> 2) {               /* No, Send 1 Byte                  */
   mlx_out_port(sccadr + 1, character);
   m032_delay();
   }
 else
   return(1);                          /* Transmit Buffer is Full !        */

 return(0);
}



/***************************************************************************/
/* Receive Character                                                       */
/*                                                                         */
/* The SCC must be initialized before !                                    */
/* Before receiving the status of the receive-buffer (RBF) is checked.     */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B											   */
/*                                                                         */
/* Result     : data = Received Character                                  */
/*                                                                         */
/* Return     : 0 OK, 1=RB not full (nothing to read)                      */
/***************************************************************************/
MLXAPI byte EXPORT m032_receive_scc_character(byte micro_slot, byte channel,
				       byte *data)
{
 byte rbf;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;

				       /* Reset External Status Interrupt  */
 m032_delay();
 mlx_out_port(sccadr, 0x10);
 m032_delay();
 rbf = mlx_in_port(sccadr) & 0x01;
 m032_delay();

				       /* Read Byte from Receive-Buffer    */
 if(rbf) {
   *data = mlx_in_port(sccadr + 1);
   m032_delay();
   }
 else return(1);

 return(0);
}



/***************************************************************************/
/* Set/Clear RTS                                                           */
/*                                                                         */
/* This procedure sets the rts-output of one sccs channel to the given     */
/* value in "rts" (0/1).                                                   */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B												*/
/*                                                                         */
/* rts        : Value to set                                               */
/***************************************************************************/
MLXAPI void EXPORT m032_set_rts(byte micro_slot, byte channel, byte rts)
{
 byte usart;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;


 usart = tmtctr[micro_slot][channel];

 if(rts) usart = usart | 0x02;

 m032_delay();
 mlx_out_port(sccadr, 5);              /* WR 5                             */
 m032_delay();
 mlx_out_port(sccadr, usart);
 m032_delay();
}



/***************************************************************************/
/* Set/Clear DTR                                                           */
/*                                                                         */
/* This procedure sets the dtr-output of one sccs channel to the given     */
/* value in "dtr" (0/1).                                                   */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B												*/
/*                                                                         */
/* dtr        : Value to set                                               */
/***************************************************************************/
MLXAPI void EXPORT m032_set_dtr(byte micro_slot, byte channel, byte dtr)
{
 byte usart;
 ushort sccadr;

 sccadr = MBASIS + CHANNEL;


 usart = tmtctr[micro_slot][channel];

 if(dtr) usart = usart | 0x80;

 m032_delay();
 mlx_out_port(sccadr, 5);              /* WR 5                             */
 m032_delay();
 mlx_out_port(sccadr, usart);
 m032_delay();
}



/***************************************************************************/
/* Read Interrupt Vector, RR0, RR1 and RR3=Pending Register                */
/*                                                                         */
/* After this procedure, the interrupt service routine must clear the      */
/* pending interrupt for the interrupting channel.                         */
/*                                                                         */
/*  status in vector         : clear pending interrupt by                  */
/*                                                                         */
/*  1) Ext./Status Interrupt : Reset Ext./Status Interrupt in WR9          */
/*  2) Transmit Buffer Empty : Write next char. into transmit buffer WR8   */
/*  3) Receive Char. Avail.  : Read character out of receive buffer RR8    */
/*  4) Special Receive Cond. : Error Reset Command in WR0                  */
/*                                                                         */
/* At the end of the interrupt service routine, a reset highest IUS in WR0 */
/* must be issued.                                                         */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* Result     :                                                            */
/*                                                                         */
/* int_vector : Interrupt Vector including Status Bits (V1..V3)            */
/*                                                                         */
/* rr0        : Read Register 0, Transmit/Receive Ext./Status Register     */
/*                                                                         */
/* rr1        : Read Register 1, Special Receive Cond. Status              */
/*                                                                         */
/* rr3        : Read Register 3, Interrupt Pending Register                */
/***************************************************************************/
MLXAPI void EXPORT m032_scc_intvec(byte micro_slot, byte *int_vector, byte *rr0, byte *rr1,
		    byte *rr3)
{
 ushort sccadr;


 m032_delay();
 mlx_out_port(MBASIS, 2);
 m032_delay();
 *int_vector = mlx_in_port(MBASIS);    /* read interrupt-vector            */
 m032_delay();

 if (*int_vector & 0x8) sccadr = MBASIS+2;
 else sccadr = MBASIS;
				       /* read RR0 (status)                */
 *rr0 = mlx_in_port(sccadr);
 m032_delay();
				       /* read RR1 (special rec. cond.)    */
 mlx_out_port(sccadr, 1);
 m032_delay();
 *rr1 = mlx_in_port(sccadr);
 m032_delay();
				       /* read RR3 (interrupt pending)     */
				       /* always read on channel A         */
 mlx_out_port(MBASIS+2, 3);
 m032_delay();
 *rr3 = mlx_in_port(MBASIS+2);
 m032_delay();

 return;
}



/***************************************************************************/
/* Get best values for to create a specific baudrate                       */
/*                                                                         */
/* This routines can be used to find the optimal parameters for FPGA and   */
/* SCC programming.                                                        */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* baudrate   : user baudrate (for example : 9600L, 19200L, ...)           */
/* quartz     : M-COM-8 quartz-oscillator-frequency (24000000L = 24MHz)    */
/*                                                                         */
/* Results    :                                                            */
/*                                                                         */
/* bauderror  : absolute error of baseclock (baseclk - desired clock)      */
/*                                                                         */
/* reg        : baseclk-register                                           */
/*                                                                         */
/* clk_mode   : SCC clock-mode (16.0, 32.0, 64.0)                          */
/*                                                                         */
/* timeconst  : SCC-timeconst counter value (for BRG, 16 Bit)              */
/*              if timeconst = -1 the BRG is not used                      */
/***************************************************************************/
MLXAPI void EXPORT m032_baudrate_calc(long baudrate, long quartz,
					 long *bauderror, byte *basereg,
					 byte *clk_mode, ushort *timeconst,
			       byte *use_brg)
{
 long base;
 long quartzfreq;

 quartzfreq = quartz;
 if (quartz == 7370000L) quartzfreq = 7372800L;
 if (quartz == 4910000L) quartzfreq = 4912500L;
 base = l_divide(quartzfreq,32);
 base = l_divide(base,baudrate);
 base = base - 2;
 *timeconst = base;
 *clk_mode = 16;
 *bauderror = baudrate - l_divide(quartzfreq,l_multi((base+2),32));
 *basereg  = 0;
 *use_brg = 1;
 return;
}

/***************************************************************************/
/* Sub-function of baudrate_calc (compares disired clock with fpga-clock)  */
/* not used in the moment, for future designs                              */
/***************************************************************************/
void EXPORT m032_check(long quartz, long base, byte clk_mode,
		ushort timeconst, byte use_brg, byte *bestx, byte *besty,
		byte *bestclk_mode, ushort *besttimeconst,
		byte *bestuse_brg,
		byte *bestpostdiv, long *bestbauderror)
{
 long err;
 long fpgaclk, help;
 byte postdivide[4] = {5, 4, 3, 2};    /* FPGA internal clock divider      */
 byte up_factor, down_factor;
 byte x, y, p;

 for(p = 0; p < 4; p++)                /* loop 1: postdiv inside FPGA      */
   {                                   /* divides quartz /5, /4, /3 or /2  */
    help = l_divide((long)quartz, (long)postdivide[p]);

    for(y = 0; y < 2; y++)             /* loop 2: y bit 0/1                */
      {                                /* divides quartz with factor       */
		 for(x = 0; x < 32; x++)         /* loop 3: x = 5 Bit Counter(0..31) */
	 {
				       /* baseclk factor inside FPGA       */
	  up_factor = ((y + 1) * x - y);
	  down_factor = ((y + 1) * x + 1);
	  if(x == 0)                   /* x = 0 && y = 0 : factor = 1      */
	    {                          /* x = 0 && y = 1 : not valid       */
	     up_factor = 1;
	     down_factor = 1;
	     if(y == 1) continue;
	    }

	  if(up_factor > 1)
		 {
	     fpgaclk = l_multi(help, (long)up_factor);
	     fpgaclk = l_divide(fpgaclk, (long)down_factor);
	    }
	  else fpgaclk = help;

	  if(fpgaclk > base) err = fpgaclk - base;
	  else err = base - fpgaclk;
				       /* abs. error                       */
	  if(err < *bestbauderror)
	    {
	     *bestbauderror  = err;
	     *bestclk_mode   = clk_mode;
		  *bestpostdiv    = postdivide[p];
	     *besttimeconst  = timeconst;
	     *bestx          = x;
	     *besty          = y;
	     *bestuse_brg   = use_brg;
	    }
	  if(err == 0L) break;

	 } // x loop (divider in fpga)
      } // y loop (divider in fpga)
   } // p loop (postdivider in fpga)
 return;
}


/***************************************************************************/
/* Set one channel (0..1) of M-COM-2 with specific baudrate.               */
/*                                                                         */
/* This routine calls m032_baudrate_calc... and checks if the M-COM-2      */
/* has enough resources to generate the baudrate.                          */
/* This is done by evaluating the EEPROM data of the module.               */
/* If so, this function sets the parameters inside the SCC and the gate-   */
/* array (using m032_init_scc).                                            */
/*                                                                         */
/* Parameters:                                                             */
/*                                                                         */
/* micro_slot : Module-Port 1..(4)..10                                     */
/*                                                                         */
/* channel    : 0=channel A, 1=channel B                                   */
/*                                                                         */
/* baudrate   : user baudrate (for example : 9600L, 19200L, ...)           */
/*                                                                         */
/* Parameters (as used in m032_init_scc())                                 */
/*                                                                         */
/* mode       : Select Clock-Mode Control (=WR11)                          */
/*                                                                         */
/* int_mode   : =Write Register 1 of a SCCs channel (WR 1, lower 5 bits)   */
/*                                                                         */
/* int_x_mode : =Write Register 15 of a SCCs channel (WR 15)               */
/*                                                                         */
/* databits   : 5, 6, 7 or 8 Databits (transmit and receive)               */
/*                                                                         */
/* stopbits   : 0=no stopbits, 1=1 stopbit, 2=1,5 stopbits, 3=2 stopbits   */
/*                                                                         */
/* parity_type: 0=no parity , 1=odd parity, 2=even parity type             */
/*                                                                         */
/* Results    :                                                            */
/*                                                                         */
/* bauderror  : absolute error of baseclock (baseclk - desired clock)      */
/*                                                                         */
/* Return     : status: 0=ok                                               */
/*                      1=channel already used (flag = 0)                  */
/*                      2=no baseclk register available (flag = 5)         */
/***************************************************************************/
MLXAPI byte EXPORT m032_set_channel(byte micro_slot, byte channel, long baudrate,
				  byte flag, byte scc_mode, byte scc_int_mode,
				  byte scc_int_x_mode, byte scc_databits,
				  byte scc_stopbits, byte scc_parity_type,
				  long *bauderror, byte *used_clkmode)
{
 byte clock_mode[3];
 ushort timeconst;
 byte   basisreg, use_brg;
 long   best_error,baud_error;
 byte   clkmode;


 flag = flag;

 clock_mode[0] = 16;                   /* SCC internal clock divider       */
 clock_mode[1] = 32;
 clock_mode[2] = 64;

 *used_clkmode = 0;



				       /* calc settings for this baudrate  */
 m032_baudrate_calc(baudrate, l_multi(m032quartz[micro_slot], 1000L), &baud_error, &basisreg,
		    &clkmode, &timeconst, &use_brg);

 *used_clkmode = clkmode;

 *bauderror = baud_error;
 best_error = baud_error;
 best_error = best_error;

 m032_init_scc(micro_slot, channel,
		  scc_mode, scc_int_mode, scc_int_x_mode,
		  timeconst, use_brg, clkmode, scc_databits, scc_stopbits,
		  scc_parity_type);

 return(0);
}


MLXAPI void EXPORT m032_define_intrp_connection(byte micro_slot, byte intrp)
{
  mlx_out_port(MBASIS+M032_PORT_C1,M032_INTERRUPT[0] [0]);
  mlx_out_port(MBASIS+M032_PORT_C2,M032_INTERRUPT[0] [1]);
  mlx_out_port(MBASIS+M032_PORT_C3,M032_INTERRUPT[0] [2]);
  mlx_out_port(MBASIS+M032_PORT_C1,M032_INTERRUPT[intrp] [0]);
  mlx_out_port(MBASIS+M032_PORT_C2,M032_INTERRUPT[intrp] [1]);
  mlx_out_port(MBASIS+M032_PORT_C3,M032_INTERRUPT[intrp] [2]);
}


static long EXPORT l_multi(long x, long y)
{
 long y1, y2, y3;

						 /* lowword(x) * lowword(y)          */
 asm mov ax, word ptr x;               /* lowword(x) in ax                 */
 asm mul word ptr y;                   /* lowword(y) * ax                  */
 asm mov word ptr y1, ax;              /* lowword(res.)=ax in lowword(y1)  */
 asm mov word ptr y1 + 2, dx;          /* highword(res.)=dx in highword(y1)*/

				       /* lowword(x) * highword(y)         */
 asm mov ax, word ptr x;               /* lowword(x) in ax                 */
 asm mul word ptr y + 2                /* highword(y) * ax                 */
 asm mov word ptr y2, 0;               /* lowword(y2) = 0 (= 16 Bit SHL)   */
 asm mov word ptr y2 + 2, ax;          /* lowword(res.) in highword(y2)    */
				       /* dx is overflow, result <= 32 Bit */

						 /* highword(x) * lowword(y)         */
 asm mov ax, word ptr x + 2;           /* highword(x) in ax                */
 asm mul word ptr y;                   /* lowword(y) * ax                  */
 asm mov word ptr y3, 0;               /* lowword(y3) = 0 (= 16 Bit SHL)   */
 asm mov word ptr y3 + 2, ax;          /* lowwrod(res.) in highword(y3)    */
				       /* dx is overflow, result <= 32 Bit */

				       /* highword(x) * highword(y)        */
				       /* not used (result = 32 Bit)       */

				       /* add all results (y1 + y2 + y3)   */
 asm mov ax,word ptr y2;               /* lowword(y2) + lowword(y1)        */
 asm add word ptr y1, ax;              /* result in lowword(y1)            */
 asm mov ax,word ptr y2 + 2;           /* highword(y2) + highword(y1) + C  */
 asm adc word ptr y1 + 2, ax;          /* result in highword(y1)           */

 asm mov ax,word ptr y3;               /* lowword(y3) + lowword(y1)        */
 asm add word ptr y1, ax;              /* result in lowword(y1)            */
 asm mov ax,word ptr y3 + 2;           /* highword(y3) + highword(y1) + C  */
 asm adc word ptr y1 + 2, ax;          /* result in highword(y1)           */

 return(y1);
}



static long EXPORT l_divide(long x, long y)
{
 long result;

 asm cmp word ptr y + 2, 0;            /* count if highword(y) > 0         */
 asm jnz count;
				       /* else divide                      */

 asm mov ax, word ptr x + 2;           /* 1. calc highword(result)         */
 asm xor dx, dx;
 asm div word ptr y;                   /* ax = highword(x) / lowword(y)    */
				       /* dx = rest                        */

 asm mov word ptr result + 2, ax;      /* save highword of result          */

 asm mov ax, word ptr x;               /* calc lowword of result           */
 asm div word ptr y;

 asm mov word ptr result, ax;          /* save lowword of result           */
 asm jmp ende;

 count:
 asm mov word ptr result, 0;
 asm mov word ptr result + 2, 0;

 test:
 asm mov ax, word ptr y;
 asm sub word ptr x, ax                /* lowword(x) -= lowword(y)         */
 asm mov ax, word ptr y + 2;
 asm sbb word ptr x + 2, ax;           /* highword(x) -= (highword(y) + c) */
 asm jc  ende;
 asm add word ptr result, 1;           /* result++                         */
 asm adc word ptr result + 2, 0;
 asm jnc test;

 ende:
 return(result);

}
