#ifndef _m032_lib_
#define _m032_lib_
/***************************************************************************/
/**                                                                       **/
/**              MODULAR-4 library for SPB module M-COM-2                 **/
/**                                                                       **/
/**                             Version 2.A.                              **/
/**                                                                       **/
/**  This file contains routines for Borland C++/3.1 to allow the user,   **/
/**  to write programs for the MODULAR-4 in "C"                           **/
/**                                                                       **/
/**             Project Start    : 02.04.1997                             **/
/**             Last change      : 02.04.1997                             **/
/**                                                                       **/
/**                         Author : H.K.                                 **/
/**                         Status : O.K.                                 **/
/**                                                                       **/
/**                                                                       **/
/** Special comments:                                                     **/
/**                                                                       **/
/** This library was tested with Borland C++ Version 3.1.                 **/
/**                                                                       **/
/** No parameters are checked.                                            **/
/**                                                                       **/
/**                                                                       **/
/**                                                                       **/
/**               (c) SORCUS Computer GmbH 1997                           **/
/**                                                                       **/
/***************************************************************************/

/* The switches for the library are set in the configuration files of Pascal
   (*.TP file) und C (project-file). They can also be set in this source file:

   Then the following #defines have to exist:

   If you use Pascal, #define BORLAND_PASCAL has to be included (default is C).

   If the library shall be used as realtime library on the MODULAR-4/486
   #define ML8 and #define ML8RT have to be included (default is PC library).

   If you work with Windows, this is detected by the compiler. No #defines have
   to be included.
*/

#include <MODULE.H>
				       /* Modul-Basisadresse                             */
#define MBASIS  micro_slot_basis[micro_slot]

#define M032_CHANNELSEL          0x10  /* Channel Select                   */
#define M032_CLKSEL              0x11  /* Clock Select Register            */
#define M032_BASECLK1            0x12  /* Base-Clock 1 Register            */
#define M032_BASECLK2            0x13  /* Base-Clock 1 Register            */
#define M032_BASECLK3            0x14  /* Base-Clock 1 Register            */
#define M032_BASECLK4            0x15  /* Base-Clock 1 Register            */
#define M032_INTSEL              0x16  /* Interrupt Select                 */
#define M032_PCLKSEL             0x17  /* PCLK Select                      */
#define M032_READ_FPGA_VERSION   0x18  /* Read FPGA-Version (8Bit)         */
#define M032_PORT_C1               0x7                          /* Interruptanwahl: Port C1                       */
#define M032_PORT_C2               0x8                          /* Interruptanwahl: Port C2                       */
#define M032_PORT_C3               0x9                          /* Interruptanwahl: Port C3                       */
#define M032_PORT_A1               0xA                          /* Konfiguration Schnittstelle A: Port A1         */
#define M032_PORT_A2               0xB                          /* Konfiguration Schnittstelle A: Port A2         */
#define M032_PORT_A3               0xC                          /* Konfiguration Schnittstelle A: Port A3         */
#define M032_PORT_B1               0xD                          /* Konfiguration Schnittstelle B: Port B1         */
#define M032_PORT_B2               0xE                          /* Konfiguration Schnittstelle B: Port B2         */
#define M032_PORT_B3               0xF                          /* Konfiguration Schnittstelle B: Port B3         */
#define M032_STATUS_DSR_A          0x8                          /* Status lesen DSR/A                             */
#define M032_STATUS_DSR_B          0xA                          /* Status lesen DSR/B                             */
#define M032_STATUS_PORT_C1        0xC                          /* Status lesen Port C1                           */

/* Zustaende der Interruptleitungen des GALS (C1, C2, C3 */

#define M032_C1_TRUE               0x1                          /* Leitung C1 = 1                                 */
#define M032_C1_FALSE              0x0                          /* Leitung C1 = 0                                 */
#define M032_C2_TRUE               0x1                          /* Leitung C2 = 1                                 */
#define M032_C2_FALSE              0x0                          /* Leitung C2 = 0                                 */
#define M032_C3_TRUE               0x1                          /* Leitung C3 = 1                                 */
#define M032_C3_FALSE              0x0                          /* Leitung C3 = 0                                 */




#if defined(__cplusplus)
extern "C" {
#endif



/* Init module library                            */
MLXAPI void EXPORT m032_bib_startup(byte micro_slot);

/* Reset SCC                                      */
MLXAPI void EXPORT m032_scc_reset(byte micro_slot);

/* Initialize SCC-Channel                         */
MLXAPI byte EXPORT m032_init_scc(byte micro_slot, byte channel,
			  byte mode, byte int_mode, byte int_x_mode,
			  ushort timeconst, byte use_brg, byte clkmode,
			  byte databits, byte stopbits, byte parity_type);

/* Read SCC-TBE Bit                               */
MLXAPI byte EXPORT m032_scc_tbe(byte micro_slot, byte channel);

/* Read SCC-RBF Bit                               */
MLXAPI byte EXPORT m032_scc_rbf(byte micro_slot, byte channel);

/* Read SCC-CTS Bit                               */
MLXAPI byte EXPORT m032_scc_cts(byte micro_slot, byte channel);

/* Read SCC-DCD Bit                               */
MLXAPI byte EXPORT m032_scc_dcd(byte micro_slot, byte channel);

/* Read SCC-Ri Bit                               */
MLXAPI byte EXPORT m032_scc_ri(byte micro_slot, byte channel);

/* Read SCC-DSR Bit                               */
MLXAPI byte EXPORT m032_scc_dsr(byte micro_slot, byte channel);

/* Send Character                                 */
MLXAPI byte EXPORT m032_transmit_scc_character(byte micro_slot, byte channel,
					byte character);

/* Receive Character                              */
MLXAPI byte EXPORT m032_receive_scc_character(byte micro_slot, byte channel,
				       byte *data);

/* Set/Clear SCC-RTS                              */
MLXAPI void EXPORT m032_set_rts(byte micro_slot, byte channel, byte rts);


/* Set/Clear SCC-DTR                              */
MLXAPI void EXPORT m032_set_dtr(byte micro_slot, byte channel, byte dtr);


/* get interrupt vector, rr0, rr1 & rr3 (pending) */
MLXAPI void EXPORT m032_scc_intvec(byte micro_slot, byte *int_vector, byte *rr0, byte *rr1,
			   byte *rr3);

/* calculate baudrate adjustments                 */
MLXAPI void EXPORT m032_baudrate_calc(long baudrate, long quartz,
			       long *bauderror, byte *basereg,
			       byte *clk_mode, ushort *timeconst, byte *use_brg);

/* sub function of baudrate_calc                  */
void EXPORT m032_check(long quartz, long base, byte clk_mode,
		       ushort timeconst, byte use_brg, byte *bestx, byte *besty,
		       byte *bestclk_mode, ushort *besttimeconst, byte *bestuse_brg,
		       byte *bestpostdiv, long *bestbauderror);

/* set channel for baudrate (incl. fpga)          */
MLXAPI byte EXPORT m032_set_channel(byte micro_slot, byte channel, long baudrate,
			     byte flag, byte scc_mode, byte scc_int_mode,
			     byte scc_int_x_mode,
			     byte scc_databits, byte scc_stopbits,
			     byte scc_parity_type, long *bauderror,
			     byte *usedclkmode);

/* Define Interrupt line of MODULAR-4/486
*/
MLXAPI void EXPORT m032_define_intrp_connection(byte micro_slot, byte intrp);

/* signed long multiplication (for compatibility) */
long EXPORT l_multi(long x, long y);

/* signed long division       (for compatibility) */
long EXPORT l_divide(long x, long y);


#if defined(__cplusplus)
} /*  End of block extern "C" */
#endif

#endif
